package p22_jaxb.programy;

import jakarta.xml.bind.JAXBContext;
import jakarta.xml.bind.JAXBException;
import jakarta.xml.bind.Unmarshaller;
import p22_jaxb.model.ArrayOfExchangeRatesTable;
import p22_jaxb.model.ExchangeRatesTable;
import p22_jaxb.model.Rate;

import java.io.File;

public class KursSredniJAXB {
    public static void main(String[] args) {
        String plik = "pliki/waluty2023.xml";
        String waluta = "EUR";
        System.out.println("Startujemy");
        long t1 = System.nanoTime();

        try {
            JAXBContext ctx = JAXBContext.newInstance(ArrayOfExchangeRatesTable.class);
            Unmarshaller u = ctx.createUnmarshaller();
            ArrayOfExchangeRatesTable array = (ArrayOfExchangeRatesTable) u.unmarshal(new File("pliki/waluty2023.xml"));
            long t2 = System.nanoTime();

            double min = Double.MAX_VALUE, max = 0, sum = 0;
            int count = 0;

            for (ExchangeRatesTable table : array.getExchangeRatesTable()) {
                for (Rate rate : table.getRates()) {
                    if(rate.getCode().equals(waluta)) {
                        count++;
                        double mid = rate.getMid().doubleValue();
                        sum += mid;
                        if(mid < min) min = mid;
                        if(mid > max) max = mid;
                        break;
                    }
                }

            }
            long t3 = System.nanoTime();
            Runtime runtime = Runtime.getRuntime();

            System.out.printf("Czas wczytywania   : %.6f s%n", (t2-t1) * 1e-9);
            System.out.printf("Czas liczenia      : %.6f s%n", (t3-t2) * 1e-9);
            System.out.printf("Czas łączny        : %.6f s%n", (t3-t1) * 1e-9);
            System.out.printf("Zajęta pamięć      : %,d B%n", runtime.totalMemory() - runtime.freeMemory());
            System.gc();
            Thread.sleep(1000);
            System.out.printf("Zajęta pamięć po gc: %,d B%n", runtime.totalMemory() - runtime.freeMemory());
            System.out.println("Count: " + count);
            if(count > 0) {
                System.out.println("Sum: " + sum);
                System.out.println("Avg: " + (sum / count));
                System.out.println("Min: " + min);
                System.out.println("Max: " + max);
            }

        } catch (JAXBException | InterruptedException e) {
            throw new RuntimeException(e);
        }
    }
}
